/*
 * libsgml -- SGML state machine parsing library.
 *                                                                  
 * Copyright (c) 2002 Uninformed Research (http://www.uninformed.org)
 * All rights reserved.
 *
 * skape
 * mmiller@hick.org
 */
#ifndef _LIBSGML_HTMLNODE_H
#define _LIBSGML_HTMLNODE_H

#include <stdio.h>

#define HTML_NODE_TYPE_ELEMENT   0x0001
#define HTML_NODE_TYPE_ATTRIBUTE 0x0002
#define HTML_NODE_TYPE_TEXT      0x0003
#define HTML_NODE_TYPE_COMMENT   0x0004

struct _html_node_list;

typedef struct _html_node {

	unsigned long     nodeType;

	char              *nodeName;
	char              *nodeValue;

	struct _html_node *attributes;

	struct _html_node *parent;

	struct _html_node *firstChild;
	struct _html_node *lastChild;

	struct _html_node *prevSibling;
	struct _html_node *nextSibling;

	unsigned char     autoclose;
	unsigned char     deferredClosure;
	unsigned char     escapeTags;

} HTML_NODE;

HTML_NODE *htmlNodeNew(unsigned long nodeType, const char *nodeName, const char *nodeValue);
void htmlNodeDestroySpecific(HTML_NODE *node);
void htmlNodeDestroy(HTML_NODE *node);

void htmlNodeAppendChild(HTML_NODE *parent, HTML_NODE *child);

void htmlNodeAppendSibling(HTML_NODE *node, HTML_NODE *sibling);

void htmlNodeRemoveNode(HTML_NODE *parent, HTML_NODE *child);

void htmlNodeSetNodeValue(HTML_NODE *node, const char *value);

HTML_NODE      *htmlNodeFindNodeByName(HTML_NODE *node, const char *name);
struct _html_node_list *htmlNodeFindNodesByName(HTML_NODE *node, const char *name);
void htmlNodeFindNodesByName_r(struct _html_node_list *nodeList, HTML_NODE *node, const char *name);

void htmlNodeOutput(HTML_NODE *node, FILE *fd);

#endif
