/*
 * libsgml -- SGML state machine parsing library.
 *                                                                  
 * Copyright (c) 2002 Uninformed Research (http://www.uninformed.org)
 * All rights reserved.
 *
 * skape
 * mmiller@hick.org
 */
#ifndef _LIBHTTP_SGMLPARSER_H
#define _LIBHTTP_SGMLPARSER_H

enum SgmlExtensionType {
	SGML_EXTENSION_TYPE_XML = 0,
	SGML_EXTENSION_TYPE_HTML,

	SGML_EXTENSION_TYPE_CUSTOM = 255
};

struct _sgml_parser;

typedef struct _sgml_handlers {

	void (*preparse)(struct _sgml_parser *parser, void *userContext);
	void (*postparse)(struct _sgml_parser *parser, void *userContext);

	void (*elementBegin)(struct _sgml_parser *parser, void *userContext, const char *elementName);
	void (*elementEnd)(struct _sgml_parser *parser, void *userContext, const char *elementName);

	void (*attributeNew)(struct _sgml_parser *parser, void *userContext, const char *attributeName, const char *attributeValue);

	void (*textNew)(struct _sgml_parser *parser, void *userContext, const char *text);

	void (*commentNew)(struct _sgml_parser *parser, void *userContext, const char *comment);

} SGML_HANDLERS;

#define SGML_STC_LETTER_TYPE_SPECIFIC   0x00
#define SGML_STC_LETTER_TYPE_SPECIFICWS 0x01
#define SGML_STC_LETTER_TYPE_NOT        0x02
#define SGML_STC_LETTER_TYPE_NOTWS      0x03
#define SGML_STC_LETTER_TYPE_ANY        0x04

#define SGML_STC_FLAG_DIVERT           (1 << 0)
#define SGML_STC_FLAG_UPDATE_STATE     (1 << 1)
#define SGML_STC_FLAG_INCL_IN_BUFFER   (1 << 2)

#define SGML_PARSER_STATE_INTEXT                      (1 << 0)

#define SGML_PARSER_STATE_INELEMENT                   (1 << 1)
#define SGML_PARSER_STATE_INELEMENTNAME               (1 << 2)
#define SGML_PARSER_STATE_INELEMENTNAME_ACTUAL        (1 << 3)
#define SGML_PARSER_STATE_INELEMENTCLOSURE            (1 << 4)
	
#define SGML_PARSER_STATE_INATTRIBUTENAME             (1 << 5)
#define SGML_PARSER_STATE_INATTRIBUTENAME_ACTUAL      (1 << 6)

#define SGML_PARSER_STATE_INATTRIBUTEVALUE_NS         (1 << 10)

#define SGML_PARSER_STATE_INATTRIBUTEVALUE_S          (1 << 12) 

#define SGML_PARSER_STATE_INCOMMENT                   (1 << 15)
#define SGML_PARSER_STATE_INCOMMENTGOTEXCLAMATION     (1 << 16)
#define SGML_PARSER_STATE_INCOMMENTGOTDASH1           (1 << 17)
#define SGML_PARSER_STATE_INCOMMENTGOTDASH2           (1 << 18)

typedef struct _sgml_state_table_rule {

	unsigned long stateIndexId;

	unsigned char letterType;
	unsigned char letter;

	unsigned long flags;

	unsigned long divertTableId;

	unsigned long isState;
	unsigned long notState;

	unsigned long addState;	
	unsigned long remState;

} SGML_STATE_TABLE_RULE;

typedef struct _sgml_state_table {

	unsigned long          stateIndexId;

	SGML_STATE_TABLE_RULE  *rules;
	unsigned long          ruleSize;

} SGML_STATE_TABLE;

typedef struct _sgml_parser {

	enum SgmlExtensionType type;

	SGML_HANDLERS          handlers;	

	SGML_STATE_TABLE       *stateTable;
	unsigned long          stateTableElements;
	SGML_STATE_TABLE_RULE  *stateTableRules;
	unsigned long          stateTableRuleElements;

	struct {

		SGML_STATE_TABLE   *currentState;

		char               *lastElementName;
		char               *lastAttributeName;

		char               *currentBuffer;
		unsigned long      currentBufferSize;

		unsigned long      state;

		void               *extensionContext;
		void               *userContext;

		void               (*onStateChange)(struct _sgml_parser *parser, unsigned long oldState, unsigned long newState);
		void               (*onDivert)(struct _sgml_parser *parser, unsigned long newIndex, unsigned long oldState, unsigned long newState, const char *lastBuffer, unsigned long lastBufferSize);

		void               (*setExtensionParam)(struct _sgml_parser *parser, void *extensionContext, unsigned long param, void *value);
		void               (*getExtensionParam)(struct _sgml_parser *parser, void *extensionContext, unsigned long param, void *value);

	} internal;

} SGML_PARSER;

SGML_PARSER *sgmlParserNew();
unsigned long sgmlParserInitialize(SGML_PARSER *parser, enum SgmlExtensionType type, SGML_HANDLERS *handlers, void *userContext);
void sgmlParserDestroy(SGML_PARSER *parser, unsigned char destroyParser);

unsigned long sgmlParserParseString(SGML_PARSER *parser, const char *string, const unsigned long stringLength);
unsigned long sgmlParserParseFile(SGML_PARSER *parser, const char *file);

void sgmlParserExtensionSetParam(SGML_PARSER *parser, unsigned long param, void *value);
void sgmlParserExtensionGetParam(SGML_PARSER *parser, unsigned long param, void *value);

void _sgmlParserInitializeStateTable(SGML_PARSER *parser);
void _sgmlParserInitializeStateTableRules(SGML_PARSER *parser);

unsigned long _sgmlParseChunk(SGML_PARSER *parser, const char *chunk, const unsigned long chunkSize);
void _sgmlParserAppendBuffer(SGML_PARSER *parser, const char *chunk, unsigned long startOffset, unsigned long length);
void _sgmlParserResetBuffer(SGML_PARSER *parser);

void _sgmlOnStateChange(SGML_PARSER *parser, unsigned long oldState, unsigned long newState);
void _sgmlOnDivert(SGML_PARSER *parser, unsigned long newIndex, unsigned long oldState, unsigned long newState, const char *lastBuffer, unsigned long lastBufferSize);

#define sgmlParserGetExtensionContext(parser) parser->internal.extensionContext
#define sgmlParserGetUserContext(parser) parser->internal.userContext

#endif
